@@ -26,6 +26,8 @@ module Numeric.Interval.Internal
   , empty
   , null
   , singleton
+  , member
+  , notMember
   , elem
   , notElem
   , inf
@@ -48,6 +50,10 @@ module Numeric.Interval.Internal
   , possibly, (<?), (<=?), (==?), (>=?), (>?)
   , idouble
   , ifloat
+  , iquot
+  , irem
+  , idiv
+  , imod
   ) where
 
 import Control.Exception as Exception
@@ -68,7 +74,8 @@ import Prelude hiding (null, elem, notElem)
 -- $setup
 
 data Interval a = I !a !a | Empty deriving
-  ( Data
+  ( Eq, Ord
+  , Data
   , Typeable
 #if __GLASGOW_HASKELL__ >= 704
   , Generic
@@ -191,10 +198,6 @@ singular Empty = False
 singular (I a b) = a == b
 {-# INLINE singular #-}
 
-instance Eq a => Eq (Interval a) where
-  (==) = (==!)
-  {-# INLINE (==) #-}
-
 instance Show a => Show (Interval a) where
   showsPrec _ Empty = showString "Empty"
   showsPrec n (I a b) =
@@ -295,7 +298,7 @@ inflate x y = symmetric x + y
 --
 -- >>> deflate 1.0 empty
 -- Empty
-deflate :: (Fractional a, Ord a) => a -> Interval a -> Interval a
+deflate :: (Num a, Ord a) => a -> Interval a -> Interval a
 deflate _ Empty               = Empty
 deflate x (I a b) | a' <= b'  = I a' b'
                   | otherwise = Empty
@@ -398,6 +401,44 @@ midpoint (I a b) = a + (b - a) / 2
 midpoint Empty = Exception.throw EmptyInterval
 {-# INLINE midpoint #-}
 
+-- | Determine if a point is in the interval.
+--
+-- >>> member 3.2 (1.0 ... 5.0)
+-- True
+--
+-- >>> member 5 (1.0 ... 5.0)
+-- True
+--
+-- >>> member 1 (1.0 ... 5.0)
+-- True
+--
+-- >>> member 8 (1.0 ... 5.0)
+-- False
+--
+-- >>> member 5 empty
+-- False
+--
+member :: Ord a => a -> Interval a -> Bool
+member x (I a b) = x >= a && x <= b
+member _ Empty = False
+{-# INLINE member #-}
+
+-- | Determine if a point is not included in the interval
+--
+-- >>> notMember 8 (1.0 ... 5.0)
+-- True
+--
+-- >>> notMember 1.4 (1.0 ... 5.0)
+-- False
+--
+-- And of course, nothing is a member of the empty interval.
+--
+-- >>> notMember 5 empty
+-- True
+notMember :: Ord a => a -> Interval a -> Bool
+notMember x xs = not (member x xs)
+{-# INLINE notMember #-}
+
 -- | Determine if a point is in the interval.
 --
 -- >>> elem 3.2 (1.0 ... 5.0)
@@ -416,9 +457,9 @@ midpoint Empty = Exception.throw EmptyInterval
 -- False
 --
 elem :: Ord a => a -> Interval a -> Bool
-elem x (I a b) = x >= a && x <= b
-elem _ Empty = False
+elem = member
 {-# INLINE elem #-}
+{-# DEPRECATED elem "Use `member` instead." #-}
 
 -- | Determine if a point is not included in the interval
 --
@@ -433,8 +474,9 @@ elem _ Empty = False
 -- >>> notElem 5 empty
 -- True
 notElem :: Ord a => a -> Interval a -> Bool
-notElem x xs = not (elem x xs)
+notElem = notMember
 {-# INLINE notElem #-}
+{-# DEPRECATED notElem "Use `notMember` instead." #-}
 
 -- | 'realToFrac' will use the midpoint
 instance Real a => Real (Interval a) where
@@ -444,27 +486,6 @@ instance Real a => Real (Interval a) where
     b = toRational rb
   {-# INLINE toRational #-}
 
-instance Ord a => Ord (Interval a) where
-  compare Empty Empty = EQ
-  compare Empty _ = LT
-  compare _ Empty = GT
-  compare (I ax bx) (I ay by)
-    | bx < ay = LT
-    | ax > by = GT
-    | bx == ay && ax == by = EQ
-    | otherwise = Exception.throw AmbiguousComparison
-  {-# INLINE compare #-}
-
-  max (I a b) (I a' b') = max a a' ... max b b'
-  max Empty i = i
-  max i Empty = i
-  {-# INLINE max #-}
-
-  min (I a b) (I a' b') = min a a' ... min b b'
-  min Empty _ = Empty
-  min _ Empty = Empty
-  {-# INLINE min #-}
-
 -- @'divNonZero' X Y@ assumes @0 `'notElem'` Y@
 divNonZero :: (Fractional a, Ord a) => Interval a -> Interval a -> Interval a
 divNonZero (I a b) (I a' b') =
@@ -886,3 +907,58 @@ ifloat = id
 -- sin 1 :: Interval Double
 
 default (Integer,Double)
+
+-- | an interval containing all x `quot` y
+-- >>> (5 `quot` 3) `member` ((4...6) `iquot` (2...4))
+-- True
+-- >>> (1...10) `iquot` ((-5)...4)
+-- *** Exception: divide by zero
+iquot :: Integral a => Interval a -> Interval a -> Interval a
+iquot i j = case (i,j) of
+  (Empty,_) -> Empty
+  (_,Empty) -> Empty
+  (I l u , I l' u') ->
+    if l' <= 0 && 0 <= u' then throw DivideByZero else I
+      (minimum [a `quot` b | a <- [l,u], b <- [l',u']])
+      (maximum [a `quot` b | a <- [l,u], b <- [l',u']])
+
+-- | an interval containing all x `rem` y
+-- >>> (5 `rem` 3) `member` ((4...6) `irem` (2...4))
+-- True
+-- >>> (1...10) `irem` ((-5)...4)
+-- *** Exception: divide by zero
+irem :: Integral a => Interval a -> Interval a -> Interval a
+irem i j = case (i,j) of
+  (Empty,_) -> Empty
+  (_,Empty) -> Empty
+  (I l u , I l' u') ->
+    if l' <= 0 && 0 <= u' then throw DivideByZero else I
+      (minimum [0, signum l * (abs u' - 1), signum l * (abs l' - 1)])
+      (maximum [0, signum u * (abs u' - 1), signum u * (abs l' - 1)])
+
+-- | an interval containing all x `div` y
+-- >>> (5 `div` 3) `member` ((4...6) `idiv` (2...4))
+-- True
+-- >>> (1...10) `idiv` ((-5)...4)
+-- *** Exception: divide by zero
+idiv :: Integral a => Interval a -> Interval a -> Interval a
+idiv i j = case (i,j) of
+  (Empty,_) -> Empty
+  (_,Empty) -> Empty
+  (I l u , I l' u') ->
+    if l' <= 0 && 0 <= u' then throw DivideByZero else I
+      (min (l `Prelude.div` max 1 l') (u `Prelude.div` min (-1) u'))
+      (max (u `Prelude.div` max 1 l') (l `Prelude.div` min (-1) u'))
+
+-- | an interval containing all x `mod` y
+-- >>> (5 `mod` 3) `member` ((4...6) `imod` (2...4))
+-- True
+-- >>> (1...10) `imod` ((-5)...4)
+-- *** Exception: divide by zero
+imod :: Integral a => Interval a -> Interval a -> Interval a
+imod i j = case (i,j) of
+  (Empty,_) -> Empty
+  (_,Empty) -> Empty
+  (_ , I l' u') ->
+    if l' <= 0 && 0 <= u' then throw DivideByZero else
+      I (min (l'+1) 0) (max 0 (u'-1))
